from ._flight_offers import FlightOffers
from ._flight_dates import FlightDates
from ._flight_destinations import FlightDestinations
from ._flight_offers_search import FlightOffersSearch
from ._hotel_offers import HotelOffers
from ._hotel_offers_by_hotel import HotelOffersByHotel
from ._hotel_offer import HotelOffer
from ._activities import Activities

__all__ = ['FlightOffers', 'FlightDates',
           'FlightDestinations', 'FlightOffersSearch',
           'HotelOffers', 'HotelOffersByHotel',
           'HotelOffer', 'Activities']



from amadeus.client.decorator import Decorator


class FlightOffersSearch(Decorator, object):
    def get(self, **params):
        '''
        Get the cheapest flights on a given journey

        .. code-block:: python

            amadeus.shopping.flight_offers_search.get(
                originLocationCode='MAD',
                destinationLocationCode='BOS',
                departureDate='2019-11-01',
                adults='1'
            )

        :param originLocationCode: the City/Airport IATA code from which
            the flight will depart. ``"MAD"``, for example for Madrid.

        :param destinationLocationCode: the City/Airport IATA code to
            which the flight is going. ``"BOS"``, for example for Boston.

        :param departureDate: the date on which to fly out, in `YYYY-MM-DD`
            format

        :param adults: the number of adult passengers with age 12 or older

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v2/shopping/flight-offers', **params)

    def post(self, body):
        '''
        Get the cheapest flights on a given journey.

        .. code-block:: python

            amadeus.shopping.flight_offers_search.post(body)

        :param body: the parameters to send to the API

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.post('/v2/shopping/flight-offers', body)



from amadeus.namespaces._reference_data import ReferenceData
from amadeus.namespaces._travel import Travel
from amadeus.namespaces._shopping import Shopping
from amadeus.namespaces._e_reputation import EReputation
from amadeus.namespaces._airport import Airport
from amadeus.namespaces._media import Media
from amadeus.namespaces._booking import Booking
from amadeus.namespaces._safety import Safety
from amadeus.namespaces._schedule import Schedule
from amadeus.namespaces._analytics import Analytics


class Core(object):
    def __init__(self):
        self.reference_data = ReferenceData(self)
        self.travel = Travel(self)
        self.shopping = Shopping(self)
        self.e_reputation = EReputation(self)
        self.airport = Airport(self)
        self.media = Media(self)
        self.booking = Booking(self)
        self.safety = Safety(self)
        self.schedule = Schedule(self)
        self.analytics = Analytics(self)



from amadeus.client.decorator import Decorator


class Seatmaps(Decorator, object):
    def get(self, **params):
        '''
        Allows you to retrieve the seat map of one or several flights based
        on the flight-orderId returned from Flight Create Orders API Call.

        .. code-block:: python

            amadeus.shopping.seatmaps.get(
                flight-orderId='<order_id>'
            )

        :param flight-orderId: identifier of the order.
        Either a flight offer or a flight order Id.

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/shopping/seatmaps', **params)

    def post(self, body):
        '''
        Allows you to retrieve the seat map of one or several flights.
        Take the body of a flight offer search or flight offer price
        and pass it in to this method to get a seatmap

        .. code-block:: python

            amadeus.shopping.seatmaps.post(body)

        :param body: the parameters to send to the API

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.post('/v1/shopping/seatmaps', body)



from amadeus.client.decorator import Decorator
from amadeus.airport._predictions import Predictions


class Airport(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.predictions = Predictions(client)



from amadeus.client.decorator import Decorator


class HotelBookings(Decorator, object):
    def post(self, hotel_offer_id, guests, payments):
        '''
        Books a hotel

        .. code-block:: python

            amadeus.booking.hotel_bookings.post(hotel_offer_id, guests, payments)

        The parameters guests and payments can be passed as dictionary
        or list of dictionaries. If they are dictionary in this method they are
        converted to a list of dictionaries.

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        guests_info = []
        payment_info = []
        if type(guests) is not list:
            guests_info.append(guests)
        else:
            guests_info.extend(guests)
        if type(payments) is not list:
            payment_info.append(payments)
        else:
            payment_info.extend(payments)
        body = {'data': {'offerId': hotel_offer_id,
                         'guests': guests_info,
                         'payments': payment_info}}
        return self.client.post('/v1/booking/hotel-bookings', body)



from amadeus.client.decorator import Decorator


class PointOfInterest(Decorator, object):
    def __init__(self, client, poi_id):
        Decorator.__init__(self, client)
        self.poi_id = poi_id

    def get(self, **params):
        '''
        Returns a single Point of Interest from a given id.

        .. code-block:: python

            amadeus.reference_data.locations.point_of_interest('9CB40CB5D0').get()

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/reference-data/locations/pois/{0}'
                               .format(self.poi_id), **params)



from amadeus.client.decorator import Decorator
from amadeus.reference_data.locations.points_of_interest._by_square \
    import BySquare


class PointsOfInterest(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.by_square = BySquare(client)

    def get(self, **params):
        '''
        Returns a list of relevant point of interests near to a given point.

        .. code-block:: python


            client.reference_data.locations.points_of_interest.get(
                longitude=2.160873,
                latitude=41.397158
            )

        :param latitude: latitude of geographic location to search around.
            For example: ``41.397158``
        :param longitude: longitude of geographic location to search around.
            For example: ``2.160873``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/reference-data/locations/pois', **params)



from amadeus.client.decorator import Decorator


class HotelOffersByHotel(Decorator, object):
    def get(self, **params):
        '''
        Get all offers for Holiday Inn Paris Notre Dame.

        .. code-block:: python

            amadeus.shopping.hotel_offers_by_hotel.get(hotelId='XKPARC12')

        :param hotelId: Amadeus Property Code (8 chars), for
            example ``XKPARC12``.

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v2/shopping/hotel-offers/by-hotel', **params)



from amadeus.client.decorator import Decorator
from .predictions import AirportOnTime


class Predictions(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.on_time = AirportOnTime(client)



from amadeus.client.decorator import Decorator
from amadeus.safety.safety_rated_locations._by_square import BySquare


class SafetyRatedLocations(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.by_square = BySquare(client)

    def get(self, **params):
        '''
        Returns the overall safety ranking and a detailed safety
            ranking of all the districts within the given radius.

        .. code-block:: python


            amadeus.safety.safety_rated_locations.get(
                longitude=2.160873,
                latitude=41.397158
            )

        :param latitude: latitude of the location to safety ranking search.
            For example: ``41.397158``
        :param longitude: longitude of the location to safety ranking search.
            For example: ``2.160873``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v1/safety/safety-rated-locations', **params)



from amadeus.client.decorator import Decorator


class TripParserStatus(Decorator, object):
    def __init__(self, client, job_id):
        Decorator.__init__(self, client)
        self.job_id = job_id

    def get(self, **params):
        '''
        Returns the parsing status and the link to the result
        in case of successful parsing.

        .. code-block:: python

            amadeus.travel.trip_parser_jobs.status('XXX').get

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get(
            '/v2/travel/trip-parser-jobs/{0}'.format(self.job_id),
            **params)



from amadeus.client.decorator import Decorator


class GeneratedPhotos(Decorator, object):
    def get(self, **params):
        '''
        Returns a link with a rendered image of a landscape

        .. code-block:: python

        amadeus.media.files.generated_photos.get(category='MOUNTAIN')

        :param category: the type of landscape to be generated,
            ``"MOUNTAIN"`` or ``"BEACH"``

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v2/media/files/generated-photos', **params)



from ._prediction import FlightChoicePrediction
from ._pricing import FlightOffersPrice
from ._upselling import Upselling

__all__ = ['FlightChoicePrediction', 'FlightOffersPrice',
           'Upselling']



from amadeus.client.decorator import Decorator
from amadeus.schedule._flights import Flights


class Schedule(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.flights = Flights(client)



from amadeus.mixins.encoder import encode_file_to_base64


# Encodes file to Base64 and constructs POST body
def from_file(file):
    encoded_file = encode_file_to_base64(file)
    return __build_trip_parser_body(encoded_file)


# Constructs POST body from Base64
def from_base64(base64):
    return __build_trip_parser_body(base64)


# Takes a Base64 and returns a dict
def __build_trip_parser_body(base64):
    return {
        'data': {
            'type': 'trip-parser-job',
            'content': base64,
        },
    }



from amadeus.client.decorator import Decorator
from amadeus.media._files import Files


class Media(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.files = Files(client)



from amadeus.client.decorator import Decorator
from amadeus.safety._safety_rated_locations import SafetyRatedLocations
from amadeus.safety._safety_rated_location import SafetyRatedLocation


class Safety(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.safety_rated_locations = SafetyRatedLocations(client)

    def safety_rated_location(self, safety_id):
        return SafetyRatedLocation(self.client, safety_id)



from amadeus.client.decorator import Decorator


class TripPurpose(Decorator, object):
    def get(self, **params):
        '''
        Predicts traveler purpose, Business or Leisure,
        with the probability in the context of search & shopping

        .. code-block:: python

            amadeus.travel.predictions.trip_purpose.get(
                            originLocationCode='NYC',
                            destinationLocationCode='MAD',
                            departureDate='2020-08-01',
                            returnDate='2020-08-12',
                            searchDate='2020-06-11')

        :param originLocationCode: the City/Airport IATA code from which
            the flight will depart. ``"NYC"``, for example for New York

        :param destinationLocationCode: the City/Airport IATA code to which
            the flight is going. ``"MAD"``, for example for Madrid

        :param departureDate: the date on which to fly out, in `YYYY-MM-DD` format

        :param returnDate: the date on which the flight returns to the origin,
            in `YYYY-MM-DD` format

        :param searchDate: the date on which the traveler performs the search,
            in `YYYY-MM-DD` format.
            If it is not specified the current date will be used

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/travel/predictions/trip-purpose', **params)



from amadeus.client.decorator import Decorator


class FlightOrder(Decorator, object):
    def __init__(self, client, flight_order_id):
        Decorator.__init__(self, client)
        self.flight_order_id = flight_order_id

    def get(self, **params):
        '''
        Retrieves a flight order based on its ID.

        .. code-block:: python

            amadeus.booking.flight_order('eJzTd9f3NjIJdzUGAAp%2fAiY=').get()

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/booking/flight-orders/{0}'
                               .format(self.flight_order_id), **params)

    def delete(self, **params):
        '''
        Deletes a flight order based on its ID.

        .. code-block:: python

            amadeus.booking.flight_order('eJzTd9f3NjIJdzUGAAp%2fAiY=').delete()

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.delete('/v1/booking/flight-orders/{0}'
                                  .format(self.flight_order_id), **params)



from amadeus.client.decorator import Decorator


class SafetyRatedLocation(Decorator, object):
    def __init__(self, client, safety_id):
        Decorator.__init__(self, client)
        self.safety_id = safety_id

    def get(self, **params):
        '''
        Returns safety information of a location by its Id.

        .. code-block:: python

            amadeus.safety.safety_rated_location('Q930400801').get()

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v1/safety/safety-rated-locations/{0}'
                               .format(self.safety_id), **params)



from amadeus.client.decorator import Decorator


class Flights(Decorator, object):

    def get(self, **params):
        '''
        Retrieves a unique flight status by search criteria

        .. code-block:: python

            amadeus.schedule.flights.get(
                            carrierCode='AZ',
                            flightNumber='319',
                            scheduledDepartureDate='2021-03-13')

        :param carrierCode: the IATA carrier code

        :param flightNumber: flight number as assigned by the carrier

        :param scheduledDepartureDate: scheduled departure date of the flight,
            local to the departure airport, format YYYY-MM-DD

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v2/schedule/flights', **params)



from ._location import Location
from ._locations import Locations
from ._airlines import Airlines
from ._recommended_locations import RecommendedLocations

__all__ = ['Location', 'Locations', 'Airlines', 'RecommendedLocations']



from amadeus.client.decorator import Decorator
from .files import GeneratedPhotos


class Files(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)
        self.generated_photos = GeneratedPhotos(client)



from amadeus.client.decorator import Decorator
from amadeus.travel.trip_parser_jobs._status import TripParserStatus
from amadeus.travel.trip_parser_jobs._result import TripParserResult


class TripParser(Decorator, object):
    def __init__(self, client):
        Decorator.__init__(self, client)

    def status(self, job_id):
        return TripParserStatus(self.client, job_id)

    def result(self, job_id):
        return TripParserResult(self.client, job_id)

    def post(self, body):
        '''
        Sends the request for the parsing with the
        booking details and input parameters.

        .. code-block:: python

            amadeus.travel.trip_parser_jobs.post(
            '{ "data": {
             "type": "trip-parser-job",
             "content": : "base64string" }}'

        You can call our helper functions with your booking details
        in a file or the content encoded in Base64

        .. code-block:: python
            amadeus.travel.trip_parser_jobs.post(amadeus.travel.from_file(path_to_file))
            amadeus.travel.trip_parser_jobs.post(amadeus.travel.from_base64(base64))

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.post('/v2/travel/trip-parser-jobs', body)



from amadeus.client.decorator import Decorator


class HotelOffer(Decorator, object):
    def __init__(self, client, offer_id):
        Decorator.__init__(self, client)
        self.offer_id = offer_id

    def get(self, **params):
        '''
        Returns details for a specific offer.

        .. code-block:: python

            amadeus.shopping.hotel_offer('XXX').get

        :rtype: amadeus.Response
        :raises amadeus.ResponseError: if the request could not be completed
        '''
        return self.client.get('/v2/shopping/hotel-offers/{0}'
                               .format(self.offer_id), **params)
